/*
	Binary File IO - Binary File Input and Output
	Copyright © 2005-2009 Laurent Cozic and Harry Whitfield

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public
	License along with this program; if not, write to the Free
	Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
	MA  02110-1301  USA
	
	Laurent Cozic contributed the C++ code and complied the code for Windows.
	
	Binary File IO - version 3.2.1
	4 August, 2009
	Copyright © 2005-2009 Laurent Cozic and Harry Whitfield
	mailto:g6auc@arrl.net
*/

/*global eprint */

/*members extractFile, join, length, platform, replace, split */

var binFileToDec = null;
var hexFileToBin = null;

function initBinaryFileIO() {
	function escapePath(s) {
		return s.replace(/([\W])/g, "\\$1");
	}
	
	if (system.platform == "macintosh") {
		binFileToDec = widget.extractFile("Code/binFileToDec");
		hexFileToBin = widget.extractFile("Code/hexFileToBin");
			
		runCommand('chmod  755 ' + escapePath(binFileToDec));
		runCommand('chmod  755 ' + escapePath(hexFileToBin));
	} else {
		binFileToDec = widget.extractFile("Code/binFileToDec.exe");
		hexFileToBin = widget.extractFile("Code/hexFileToBin.exe");
	}
}

function readBinaryFile(path, start, count)	{		// Returns a space separated list of decimal integers
													// in the range 0 to 255 with a -1 at the end.
													// The file is opened in read-only mode;
													// count bytes are read from position start.

	function escapePath(s) {
		return s.replace(/([\W])/g, "\\$1");
	}
	
	if (binFileToDec) { 
		var cmd;
		
		if (system.platform == "windows") {
			cmd = '"' + binFileToDec + '" "' + path + '" ' + start + ' ' + count;
		} else {
			cmd = escapePath(binFileToDec) + ' ' + escapePath(path) + ' ' + start + ' ' + count;
		}		
		//eprint(cmd);
		return runCommand(cmd);
	} else {
		beep();
		alert('readBinaryFile() is not implemented.');
		return "-1";
	}
}

function writeBinaryFile(path, start, count, data) {// data should be a space separated list of decimal integers
													// in the range 0 to 255 with an optional -1 at the end.
													// If start is zero, the file is opened/created and (over)written from the beginning,
													// otherwise, the file is opened and overwritten at position start.

	function escapePath(s) {
		return s.replace(/([\W])/g, "\\$1");
	}

	var hex = "0123456789ABCDEF";
	var dataArr = data.split(' ');
	var hexData = "";
	var c;
	for (var i = 0; i < dataArr.length; i += 1) {
		if ((c = dataArr[i]) < 0) {
			break;
		}
		hexData += hex[(c >> 4) & 15] + hex[c & 15];
	}

	if (hexFileToBin) { 
		var cmd;
		
		if (system.platform == "windows") {
			cmd = '"' + hexFileToBin + '" "' + path + '" ' + start + ' ' + count + ' ' + hexData;
		} else {
			cmd = escapePath(hexFileToBin) + ' ' + escapePath(path) + ' ' + start + ' ' + count + ' ' + hexData;
		}
		//eprint(cmd);
		return runCommand(cmd);
	} else {
		beep();
		alert('writeBinaryFile() is not implemented.');
		return "-1";
	}
}

function copyBinaryFile(path, outPath) {	// shows how to handle large files
	var start = 0;
	var count = 8192;													// block size
	var data = readBinaryFile(path, start, count).split(' ');			// read (up to) block size
	if (data[0] < 0) {
		return;
	}										// no data
	var res = writeBinaryFile(outPath, start, count, data.join(' '));	// open/create file and write (up to) count bytes of data at beginning

	while (data.length > count)	{										// more data expected
		start = start + count;
		data = readBinaryFile(path, start, count).split(' ');			// read (up to) block size
		if (data[0] < 0) {
			return;
		}
		res = writeBinaryFile(outPath, start, count, data.join(' '));	// open file and write (up to) count bytes of data at "start"
	}
	//eprint(res);
}

function testBinaryFile(path) {	// check that all binary values can be correctly written
	var data = [];
	for (var i = 0; i < 512; i += 1) {
		var b = i & 255;
		data[i] = b;
	}
	data[data.length] = -1;
	writeBinaryFile(path, 0, 512, data.join(' '));			// open/create file and write 512 bytes of data at beginning
	//eprint(res);
}

initBinaryFileIO();

//testBinaryFile("/Users/nhw/Desktop/testBinary.bin");
