/*
	Hex Display - A Hex Display Widget
	Copyright © 2008-2009 Harry Whitfield

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public
	License along with this program; if not, write to the Free
	Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
	MA  02110-1301  USA
	
	Hex Display - version 1.0.1
	4 August, 2009
	Copyright © 2008-2009 Harry Whitfield
	mailto:g6auc@arrl.net
*/

/*global buildVitality, readBinaryFile, frm, makeDisplay, theText, copyBinaryFile, testBinaryFile,
		 selectEntry, theInput, mainWindow, gPath
*/

/*members addressWidthPref, data, floor, fromCharCode, getFileInfo, 
    hexItemWidthPref, indexOf, isDirectory, itemExists, keyIdentifier, 
    keyString, lastIndexOf, length, match, onKeyDown, onKeyPress, prototype, 
    push, rejectKeyPress, replace, round, scrollX, scrollY, shiftKey, size, 
    split, substring, toLowerCase, toString, toUpperCase, tooltip, value, 
    writeFile
*/

var BLOCKSIZE = 2048;
var dockImage = "Resources/vitality.png";
var gStartAddr = 0, gEndAddr = 0, gEndOfFile = null;
var gPattern = [], gFoundAddress = 0;
var bytes = [];
var dataBase = null;

Array.prototype.indexOf = function (subArray, start) {
	if (start === undefined) {
		start = 0;
	}
	var m = subArray.length, n = this.length - m - 1;
	outer:
	for (var i = start; i < n; i += 1) {
		if (this[i] == subArray[0]) { 
			for (var j = 1; j < m; j += 1) {
				if (this[i + j] != subArray[j]) {
					continue outer;
				}
			}
			return i;
		}
	}
	return -1;
};

function hexAddr(addr) {
	var hex = addr.toString(16).toUpperCase();
	while (hex.length < 6) {
		hex = '0' + hex;
	}
	return hex;
}

function vitality(start, end) {
	gStartAddr = start;
	gEndAddr = end;
	if (start != end) {
		buildVitality(dockImage, hexAddr(start), hexAddr(end));
	}
	else if (start === 0) {
		buildVitality(dockImage, "No Data", "");
	} else  {
		buildVitality(dockImage, "", "No File");
	}
}

function dockOpened() {
	vitality(gStartAddr, gEndAddr);
}

function hexDump(path, start, count) {
	dataBase = [];

	if (gPath === null) {
		vitality(-1, -1);
		beep();
		return "";
	}
	
	var fileInfo = filesystem.getFileInfo(path);
	if (fileInfo.isDirectory)  {
		vitality(0, 0);
		beep();
		return "";
	}

	var data = readBinaryFile(path, start, count);
	if (!data) {
		vitality(0, 0);
		beep();
		return "";
	}

	gEndOfFile = fileInfo.size; //eprint('size: ' + gEndOfFile + '==' + hexAddr(gEndOfFile));
	gFoundAddress = 0;	

	frm.scrollX = frm.scrollY = 0;

	bytes = data.split(' ');
	var n = bytes.length;
	if (Number(bytes[n - 1]) < 0) {
		n -= 1;
	}
	var extra = n % 16;
	var lines = (n - extra) / 16;
	vitality(start, start + n);
	var ss   = "";
	var line = "";
	
	function printTextLine(i, k) {
		var s = "";
		for (var j = 0; j < k; j += 1) {
			var c = Number(bytes[16 * i + j]);
			if ((c > 31) && (c < 128)) {
				s += String.fromCharCode(c);
			} else {
				s += ".";
			}
		}
		return s;
	}

	function printLine(i) {
		var addr = (16 * i + start).toString(16).toUpperCase();
		while (addr.length < 6) {
			addr = '0' + addr;
		}
		var s = addr + ": ";
		for (var j = 0; j < 16; j += 1) {
			var hex = Number(bytes[16 * i + j]).toString(16).toUpperCase();
			if (hex.length < 2) {
				hex = '0' + hex;
			}
			s += hex + ' ';
		}
		s += ' ';
		s += printTextLine(i, 16);
		s += '\n';
		return s;
	}

	function printLastLine(i, extra) {
		var j;
		if (extra === 0) { 
			return "";
		}
		var addr = (16 * i + start).toString(16).toUpperCase();
		while (addr.length < 6) {
			addr = '0' + addr;
		}
		var s = addr + ": ";
		for (j = 0; j < extra; j += 1) {
			var hex = Number(bytes[16 * i + j]).toString(16).toUpperCase();
			if (hex.length < 2) {
				hex = '0' + hex;
			}
			s += hex + ' ';
		}
		s += ' ';
		if (extra !== 0) {
			for (j = extra; j < 16; j += 1) {
				s += '   ';
			}
		}
		s += printTextLine(i, extra);
		s += '\n';
		return s;
	}

	for (var i = 0; i < lines; i += 1) {
		line = printLine(i);
		dataBase[i] = line;
		ss += line;
	}
	line = printLastLine(i, extra);
	if (line !== "") {
		dataBase[i] = line;
	}
	ss += line;

	makeDisplay(dataBase);

	return ss;
}

function goHome() {
	theText.data = hexDump(gPath, 0, BLOCKSIZE);
}

function goDown() {
	var startAddr = gStartAddr + BLOCKSIZE;
	if (startAddr >= gEndOfFile) {
		beep();
		return;
	}
	theText.data = hexDump(gPath, startAddr, BLOCKSIZE);
}

function goUp() {
	if (gStartAddr === 0)  {
		beep();
		return;
	}
	var startAddr  = gStartAddr - BLOCKSIZE;
	if (startAddr < 0) {
		startAddr = 0;
	}
	theText.data = hexDump(gPath, startAddr, BLOCKSIZE);
}

function goEnd() {
	var endAddr = gEndOfFile;
	var rem     = endAddr % 16;
	if (rem !== 0) {
		endAddr += 16 - rem;
	}
	var startAddr  = endAddr - BLOCKSIZE;
	if (startAddr < 0) {
		startAddr = 0;
	}
	theText.data = hexDump(gPath, startAddr, BLOCKSIZE);
}

function xtn(s) {
    var ext;
    var idx = s.lastIndexOf(".");
    if (idx >= 0) {
        ext = s.substring(idx);
        if (ext.length > 7) {
            return "";
        }
        if (ext.length > 1) {
            return ext;
        }
    }
    return "";
}

function saveFile() {
	if (gPath === null) {
		beep();
		return;
	}
	if ((filesystem.itemExists(gPath)) && (!filesystem.isDirectory(gPath))) {
		var extn = xtn(gPath);
		var outPath = saveAs();
		if (outPath) {
			if (xtn(outPath) === "") {
				outPath += extn;
			}
			copyBinaryFile(gPath, outPath);
		}
	} else {
		beep();
		return;
	}
}

function testFile() {
	var outPath = saveAs();
	if (outPath) {
		if (xtn(outPath) === "") {
			outPath += ".bin";
		}
		testBinaryFile(outPath);
	}
}

function dumpFile() {
	if (theText.data !== "") {
		var outPath = saveAs();
		if (outPath) {
			if (xtn(outPath) === "") {
				outPath += ".txt";
			}
			filesystem.writeFile(outPath, theText.data);
		}
	} else {
		beep();
		return;
	}
}

function forward(s) {
	var delta = parseInt(s, 16);
	if (isNaN(delta)) {
		beep();
		return;
	}
	var rem = delta % 16;
	if (rem !== 0) {
		delta -= rem;
	}
	var startAddr  = gStartAddr + delta;
	if (startAddr >= gEndOfFile) {
		beep();
		return;
	}
	theText.data = hexDump(gPath, startAddr, BLOCKSIZE);
}

function backward(s) {
	var delta = parseInt(s, 16);
	if (isNaN(delta)) {
		beep();
		return;
	}
	var rem = delta % 16;
	if (rem !== 0) {
		delta += 16 - rem;
	}
	var startAddr  = gStartAddr - delta;
	if (startAddr < 0) {
		startAddr = 0;
	}
	theText.data = hexDump(gPath, startAddr, BLOCKSIZE);
}

function getPattern(s) {
	var lookFor = /^(ind)?((\s*[0-9A-Fa-f]{2})+)\s*/i;
	var found = s.match(lookFor);
	if (found) {
		s = found[2].replace(/\s/g, "");
		var pattern = [];
		for (var i = 0; i < s.length; i += 2) {
			pattern.push(parseInt(s[i] + s[i + 1], 16));
		}
		return pattern;
	} else {
		return [];
	}
}

function barHOffset(addr) {
	var addressWidth = Number(preferences.addressWidthPref.value);
	var hexItemWidth = Number(preferences.hexItemWidthPref.value);
	return Math.round(addressWidth + 0.5 * hexItemWidth * (addr % 16));
}

function barWidth(length) {
	var hexItemWidth = Number(preferences.hexItemWidthPref.value);
	return Math.round(0.5 * hexItemWidth * length);
}

function find(s) {
	var pattern = getPattern(s);
	gPattern = pattern;
	if ((pattern.length === 0) || (bytes.length === 0)) {
		beep();
		return;
	}
	var addr = bytes.indexOf(pattern);
	if (addr >= 0) {
		gFoundAddress = addr;
		var line = Math.floor(addr / 16);
		selectEntry(line, barHOffset(addr), barWidth(pattern.length), "#FFFFFF", "#0000FF");
		frm.scrollY = -16 * line;
	} else {
		beep();
	}
}

function findNext() {
	var pattern = gPattern;
	if ((pattern.length === 0) || (bytes.length === 0)) {
		beep();
		return;
	}
	var addr = bytes.indexOf(pattern, gFoundAddress + 1);
	if (addr >= 0) {
		gFoundAddress = addr;
		var line = Math.floor(addr / 16);
		selectEntry(line, barHOffset(addr), barWidth(pattern.length), "#FFFFFF", "#0000FF");
		frm.scrollY = -16 * line;
	} else {
		beep();
	}
}

function goAddress(s) {
	var lookFor = /^\s*([0-9A-Fa-f]+)\s*$/;
	var found = s.match(lookFor);
	if (!found) {
		beep();
		return;
	}
	var address = parseInt(found[1], 16);
	if (isNaN(address)) {
		beep();
		return;
	}
	var rem = address % 16;
	if (rem !== 0) {
		address -= rem;
	}
	var startAddr  = address;
 	if (startAddr < 0) {
 		beep();
 		return;
 	}
 	if (startAddr >= gEndOfFile) {
 		beep();
 		return;
 	}
	theText.data = hexDump(gPath, startAddr, BLOCKSIZE);
}

function commandMessage(s) {
	if (s !== "") {
		s = s.toLowerCase();
		switch (s[0]) {
		case "h":
			goHome();
			break;
		case "d":
			goDown();
			break;
		case "u":
			goUp();
			break;
		case "e":
			goEnd();
			break;
		case "+":
			forward(s.substring(1));
			break;
		case "-":
			backward(s.substring(1));
			break;
		case "f":
			find(s.substring(1));
			break;
		case "g":
			findNext();
			break;
		case "s":
			saveFile();
			break;
		case "t":
			testFile();
			break;
		case "w":
			dumpFile();
			break;
		default:
			goAddress(s);
			break;
		}
	}
}

function keyPress(event) {
	var keyString = event.keyString;
	if ((keyString == "Return") || (keyString == "Enter")) {
		theInput.rejectKeyPress();
		commandMessage(theInput.data);
		if ((keyString == "Return") && (!event.shiftKey)) {
			theInput.data = "";
		}
	}
}

function keyDown(event) {
	var keyIdentifier = event.keyIdentifier;
	switch (keyIdentifier) {
	case "Home":
		goHome();
		break;
	case "PageDown":
		goDown();
		break;
	case "PageUp":
		goUp();
		break;
	case "End":
		goEnd();
		break;
	default:
		break;
	}
}

theInput.onKeyPress  = keyPress;
theInput.onKeyDown   = keyDown;
mainWindow.onKeyDown = keyDown;

theInput.tooltip = "Commands are h(ome), d(own), u(p), e(nd),\n+<hex>, -<hex>, <hex>,\nf(ind), g(et next), s(ave), t(est), w(rite)";
vitality(-1, -1);
